@@ -25,8 +25,12 @@ def get_args():
     CML Parser
     '''
     parser = argparse.ArgumentParser(description='Auto-Wannier90-Fit. Using minimize method to choose the most suitable energy windows.')
-    parser.add_argument('--path', default='.',
+    parser.add_argument('--path', action='store', type=str, default='.',
                         help='Default: .')
+    parser.add_argument('--environ', action='store', type=str, default=None,
+                        help='Full Environment Variables from `os.environ`. Default: None')
+    parser.add_argument('--debug', action='store_true', default=False,
+                        help='Change config level for logging to `DEBUG`. Defulat is `INFO`.')
     args = parser.parse_args()
     return args
 
@@ -40,8 +44,8 @@ def main_task(opt_input:ArrayLike, w90:W90) -> float:
         opt_input = w90.rational_opt_input(opt_input)
         w90.show_total_dis_dict(opt_input)
 
-    opt_input_dict = w90.opt_dis_dict(opt_input)
-    w90.edit_win(opt_input_dict)
+    input_dict = w90.total_dis_dict(opt_input)
+    w90.edit_win(input_dict)
 
     # TODO: run job on local machine (or without queueing system) through `wannier90.x`. Check outputs in `wannier90.wout` file or `wannier90_band.dat` file?
     job = Job(w90.config)
@@ -51,8 +55,10 @@ def main_task(opt_input:ArrayLike, w90:W90) -> float:
     job.check_run_until_stop()
     logger.info('Job <{0}> is done.'.format(w90.config.job_name).ljust(80, ' '))
 
-    mtime = time.ctime(os.path.getmtime(w90.config.w90_bnd))
-    logger.info(f'`{w90.config.w90_bnd}` was last modified at {mtime}.'.ljust(80, ' '))
+    w90_bnd = os.path.join(w90.path, w90.config.w90_bnd)
+    mtime = time.ctime(os.path.getmtime(w90_bnd))
+    logger.info(f'{os.path.abspath(w90_bnd)}'.ljust(80, ' '))
+    logger.info(f'    was last modified at {mtime}.'.ljust(80, ' '))
     logger.info('Evaluating the quality ...'.ljust(80, ' '))
 
     res = w90.evaluate()
@@ -67,7 +73,8 @@ if __name__ == '__main__':
 
     strtime = time.strftime(r'%Y%m%d%H%M%S', time.localtime())
     logfilename = 'log_autow90_{0}.log'.format(strtime)
-    logging.basicConfig(level=logging.INFO,     # logging.DEBUG
+    level = logging.DEBUG if args.debug else logging.INFO
+    logging.basicConfig(level=level,
                         filename=os.path.join(args.path, logfilename),
                         datefmt='%H:%M:%S',
                         format='%(message)s ! %(asctime)s/%(levelname)s/%(module)s/%(lineno)d')
@@ -78,11 +85,12 @@ if __name__ == '__main__':
                      ░█▄▄█ █  █   █ ──█──█─░█░█░█ ▀▄▄█ █▄▀█
                      ░█ ░█ ─▀▀▀───▀─  ▀▀▀▀ ░█▄▀▄█──▄▄▀ █▄▄█
                      
-    Module for automatically dis energy window optimization inside `pyw90`.
+    A module inside `pyw90` for automatically optimizing dis energy windows.
     For more information, please refer to https://github.com/Cloudiiink/pyw90
                                                                                 """)
 
-    config = Config(yaml_file= os.path.join(args.path, "auto_w90_input.yaml"))
+    config = Config(yaml_file= os.path.join(args.path, "auto_w90_input.yaml"),
+                    environ=eval(args.environ))
     config.info()
     w90 = W90(config, path=args.path)
 
