@@ -6,6 +6,7 @@ from typing import Tuple
 import warnings
 from collections import Counter
 from functools import reduce
+import os
 
 # Calculate the percentage of given sites and given orbitals inside the energy interval
 from scipy.interpolate import splrep, splev
@@ -21,6 +22,7 @@ matplotlib.use('Agg')
 
 # pyw90
 from pyw90.utility.utility import num2str, str2num, get_id_from_specie
+from pyw90.utility.utility import bc
 
 def check_orb_name(func):
     def __wrapper__(*args, **kwargs):
@@ -229,11 +231,10 @@ class DOS():
         return '\n'.join(l_w90_str)
 
     @classmethod
-    def parse_projections_from_selected(cls, s: str, delimiter:str=';') -> set[str]:
+    def parse_projections_from_selected(cls, s: str) -> set[str]:
         r"""
         Convert input string to `set` with `key_string` format (`spec`_`struc_id`_`orb_name`).
-        
-        The default delimiter is `;` and we can use `1-4|6` to represent `1,2,3,4,6` both in site_id and orbital_id.
+        We  use `1-4|6` to represent `1,2,3,4,6` both in site_id and orbital_id.
 
         **Note**: `pymatgen` notation used in returned `key_string`.
         """
@@ -246,8 +247,8 @@ class DOS():
             spec, pos, orbs = l.split(',')
             pos, orb = str2num(pos), str2num(orbs) 
             for p in pos:
-                for orb in orb:
-                    res.append(f'{spec}_{p}_{cls.orbital_names[orb]}')
+                for o in orb:
+                    res.append(f'{spec}_{p}_{cls.orbital_names[o]}')
         return set(res)
 
     @staticmethod
@@ -289,8 +290,8 @@ class DOS():
 
         # TODO: use `PROCAR` to generate a result, although it might not be accurate enough.
         if left < min(ee) or right > max(ee):
-            warnings.warn(f'CHECK YOUR INPUT! The energies in `EIGENVAL` is ranged from {ee.min()} to {ee.max()}' \
-                        ' which not include all the energy ranged from {left} to {right} you input.')
+            warnings.warn(f'CHECK YOUR INPUT! The energies in `EIGENVAL` is ranged from {ee.min()} to {ee.max()}, ' \
+                          f'which does not include all the energy ranged from {left} to {right} you input.')
 
         lspec, lstruc, lorbid, lorbname, lstr, ldos = [list() for _ in range(6)]
         for i in range(num_sites):
@@ -426,13 +427,20 @@ class DOS():
         # color = [colors[0] if row['key_string'] in selected else colors[-1]
         #              for _, row in df.iterrows()]
         color = [colors[0] if row in selected else colors[-1] for row in df['key_string']]
-        print(f'Plot with selected orbitals in `{colors[0]}` and not selected orbitals in `{colors[-1]}`.')
-        print(f'{len(selected)} orbitals selected')
-        mask = (np.array(color) == colors[0])
-        print(f'Plot {len(df)} orbitals with {sum(mask)} selected.')
+        mask  = (np.array(color) == colors[0])
+        bc.cprint(bc.BLUE, f'Plotted with selected orbitals in `{colors[0]}` and non-selected orbitals in `{colors[-1]}`.')
+        bc.cprint(bc.BLUE, f'Plotted with a total of {len(df)} orbitals, {sum(mask)} of which are selected.')
+        bc.cprint(bc.BLUE, f"Figure should be stored at {os.path.abspath(savefig)}")
         
-        ax = df.plot.barh(x="key_string", y="dos", color=color) #, figsize=(8, 20))
+        import matplotlib.patheffects as pe
+
+        # dynamic figsize and font support
+        plt.rcParams['font.family'] = "Open Sans"
+        ax = df.plot.barh(x="key_string", y="dos", color=color, figsize=(8, len(df)/32*8))
         ax.set_axisbelow(True)
+        label = ax.bar_label(ax.containers[0], fmt='%.3f', padding=5)
+        for l in label:
+            l.set_path_effects([pe.withStroke(linewidth=8, foreground="w")])
         plt.grid(axis='x')
 
         # save figure in local folder not in source folder
