@@ -9,6 +9,7 @@ from scipy import interpolate
 from scipy.signal import savgol_filter
 
 from pyw90.lib.config import Config
+from pyw90.utility.utility import bc
 
 import logging
 logger = logging.getLogger(__name__)
@@ -17,7 +18,9 @@ class W90():
     r'''
     This is main class of `pyw90` and contains necessary `dis_windows` parameters for `Wannier90`. The class also offers methods including `dis_window` suggestion and evaluating the quality of Wannier Functions.
     '''
-    def __init__(self, config:Config=None, eig:str=None, win:str='wannier90.win', efermi=0, nwann=0, path:str='.', nbnds_excl:int=None, ndeg:int=1, eps: float=4.0e-3):
+    def __init__(self, config:Config=None, eig:str=None, win:str='wannier90.win', 
+                 vasp_bnd="bnd.dat", seedname="wannier90",
+                 efermi=0, nwann=0, path:str='.', nbnds_excl:int=None, ndeg:int=1, eps: float=4.0e-3):
         '''
         Init
         
@@ -28,17 +31,23 @@ class W90():
         '''
         self.config = config
         if self.config != None:
-            self._win   = config.win
-            self.efermi = config.efermi
-            self.nwann  = config.nwann
+            self.win      = config.win
+            self.efermi   = config.efermi
+            self.nwann    = config.nwann
+            self.path     = config.path     # the directory containing the input file
+            self.vasp_bnd = config.vasp_bnd
+            self.w90_bnd  = config.w90_bnd
+            self.seedname = self.win[:-4]
         else:
-            self._win   = win
-            self.efermi = efermi
-            self.nwann  = nwann
+            self.win      = win
+            self.efermi   = efermi
+            self.nwann    = nwann
+            self.path     = os.path.abspath(path)     # the directory containing the input file
+            self.vasp_bnd = vasp_bnd
+            self.seedname = seedname
+            self.w90_bnd  = seedname + '_band.dat'
 
-        self._sys   = self._win[:-4]
-        self.eig = self._sys + '.eig' if eig == None else eig
-        self.path = os.path.abspath(path)     # the directory containing the input file
+        self.eig = self.seedname + '.eig' if eig == None else eig
         self.nbnds_excl = nbnds_excl
         self.ndeg   = ndeg      # denegeracy of bands, actually need to set 2 only meets Kramers degeneracy.
         self.eps    = eps     # tolerance for generate `dis_windows`
@@ -49,7 +58,7 @@ class W90():
 
     def read_eigenval(self):
         r'''
-        Read band energies from VASP EIGENVAL file or `self._sys.eig` file.
+        Read band energies from VASP EIGENVAL file or `self.seedname.eig` file.
         '''
         if self.eig[-3:] == 'eig':
             self.nspin = 1
@@ -97,9 +106,9 @@ class W90():
 
     def read_wannier90_win(self):
         r'''
-        Read parameters from `self._win` file.
+        Read parameters from `self.win` file.
         '''
-        with open(os.path.join(self.path, self._win)) as f:
+        with open(os.path.join(self.path, self.win)) as f:
             dat = [line.strip() for line in f if line.strip()]
 
         for l in dat:
@@ -151,31 +160,32 @@ class W90():
                 'Begin Kpoint_Path\n' \
                 'End Kpoint_Path\n')
         elif flag == 'basic':
-            print('num_wann  = 8\n' \
+            print('num_wann  = 6\n' \
                 '# num_bands = 15\n' \
                 '\n' \
+                'exclude_bands : 1-5\n' \
+                '\n' \
                 'begin projections\n' \
-                'Ga:l=0;l=1\n' \
-                'As:l=0;l=1\n' \
+                'Ga:l=1\n' \
+                'As:l=1\n' \
                 'end projections\n' \
                 '\n' \
                 '# spinors = .true.\n')
         else:
             print('Unknown flag: ', flag)
 
-    def plot_eigenval(self, erange:Tuple[float,float]=None, separate:bool=False, savefig:str='eigenval_dis.png'):
+    def plot_eigenval(self, erange:Tuple[float,float]=None, separate:bool=False, 
+                      savefig:str='eig_dist.pdf'):
         r'''
         Plot the eigenvalue distribution of each bands. When `separate` is `False`, we will merge the distribution of bands when there is no global gap.
 
         ### Parameters
         - `erange`: The energy interval you concerned about.
         - `separate`: Control whether to merge the bands without the global gap or not. Default is `False`.
-        - `savefig`: The file name of saved image result.
+        - `savefig`: The file name of saved image result. default: eig_dist.pdf
         '''
         plt.rcParams['font.family'] = "Open Sans"
 
-        fig, ax = plt.subplots()
-
         def label_bar(string, height, rect):
             """Attach a text label on top of bar."""
             ax.annotate(f'{string}',
@@ -198,30 +208,41 @@ class W90():
                 idx_min, idx_max = [0]+list(idx+1), list(idx)+[self.nbnds-1]
             
             eplot_min, eplot_max = self.eband_min[idx_min], self.eband_max[idx_max]
-
             ymin, ymax = erange if erange else (-1e4, 1e4)
+            N = np.sum(np.logical_and(eplot_max > ymin, eplot_min < ymax))
+            fig, ax = plt.subplots(figsize=(N/2, 6))
 
+            xticks = []
             for i, (emin, emax) in enumerate(zip(eplot_min, eplot_max)):
                 if emax > ymin and emin < ymax:
-                    rect = ax.bar(i, emax-emin, width=0.6, bottom=emin-self.efermi, color='b')
-                    labeli = ax.bar_label(rect, labels=[f'{idx_min[i]-self.nbnds_excl}'], padding=5)
+                    xticks.append(2*i)
+                    rect = ax.bar(2*i, emax-emin, width=0.6, bottom=emin-self.efermi, color='b')
+                    labeli = ax.bar_label(rect, labels=[f'{idx_min[i]}-{idx_max[i]}'], padding=5)
                     for l in labeli:
                         l.set_path_effects([pe.withStroke(linewidth=8, foreground="w")])
-                    # label_bar(f'{idx_min[i]-self.nbnds_excl}', emax, rect[0])
+
+            ax.set_xticks(xticks, labels=[f'G{i//2}' for i in xticks])
 
         # dont merge bands without global gap
         else:
-            savefig = savefig.split('.')[0] + '_separate.png'
+            savefig = savefig[:-4] + '_separate.pdf'
             idx = np.arange(self.nbnds_excl, self.nbnds-1, self.ndeg) if self.nbnds_excl else np.arange(0, self.nbnds-1, self.ndeg)
             ymin, ymax = erange if erange else (-1e4, 1e4)
+            N = np.sum(np.logical_and(self.eband_max > ymin, self.eband_min < ymax))
+            if N > 50:
+                bc.cprint(bc.RED, "WARNING: Since there are two many bands to plot" \
+                                  "the figure might become very long!")
+            fig, ax = plt.subplots(figsize=(N/16*4, 6))
             for i in idx:
                 emin, emax = self.eband_min[i], self.eband_max[i]
                 if emax > ymin and emin < ymax:
-                    rect = ax.bar(i, emax-emin, width=0.6, bottom=emin-self.efermi, color='b')
-                    labeli = ax.bar_label(rect, labels=[f'{i-self.nbnds_excl}'], padding=5)
-                    for l in labeli:
-                        l.set_path_effects([pe.withStroke(linewidth=8, foreground="w")])
-                    # label_bar(f'{i-self.nbnds_excl}', emax, rect[0])
+                    if i % 5 != 0:
+                        rect = ax.bar(i, emax-emin, width=0.6, bottom=emin-self.efermi, color='b')
+                    else:
+                        rect = ax.bar(i, emax-emin, width=0.6, bottom=emin-self.efermi, color='k')
+                        labeli = ax.bar_label(rect, labels=[f'{i}'], padding=5)
+                        for l in labeli:
+                            l.set_path_effects([pe.withStroke(linewidth=8, foreground="w")])
 
         ax.set(ylabel='Energy / eV')
         ax.set_axisbelow(True)
@@ -334,7 +355,7 @@ class W90():
         mask = self.eband_min >= emin
         idx = np.argmax(mask) + nwann - 1
         if idx >= self.nbnds:
-            print(f'There is no enough states for {emin} with {nwann} WFs!')
+            bc.cprint(bc.RED, f'There is no enough states for {emin} with {nwann} WFs!')
             res = int(self.emax) + 1.
         else:
             res = self.eband_max[idx] + self.eps
@@ -381,19 +402,15 @@ class W90():
         - `erange`: The energy interval you concerned about.
         '''
         N = self.count_states_most(erange)
-        print(f'There are {N} states at most in {erange} with Fermi level at {self.efermi}.')
         emin, emax = min(erange), max(erange)
         dN = N - self.nwann
         froz_min_list, froz_max_list, N = [], [], []
 
         if self.nwann <= 0:
-            print(f'Please input vaild number of WF, now is {self.nwann}.')
+            bc.cprint(bc.RED, f'Please input vaild number of WF, now is {self.nwann}.')
             return pd.DataFrame(columns=['dis_froz_min', 'dis_froz_max', 'N'])
 
         elif dN > 0:
-            print('Suggest dis_froz_min & dis_froz_max as following:')
-            print(f'nwann: {self.nwann}    degenercy: {self.ndeg}    Fermi: {self.efermi:12.6f}')
-
             for i in range(0, dN + 1, self.ndeg):
                 # First get froz_max for nwann = nwann_input + i, then get froz_min for nwann = nwann_input and froz_max
                 froz_max = self.suggest_froz_max(emin, nwann=self.nwann+i)
@@ -409,7 +426,10 @@ class W90():
                                  columns=['dis_froz_min', 'dis_froz_max', 'N'])
 
             if num_missing > 0:
-                print(f'\nWANRING: There are {num_missing} states between given `emin`: {emin} and lowest `dis_froz_min`: {min(froz_min_list)}. \n\nPlease carefully treat the suggestion of dis_froz_min / dis_froz_max and check energy range of each bands again. This situation usually happens in no-SOC system with many denegeracy point. But we still want to give you some useful energy window information with states less than number of WFs.\n')
+                bc.cprint(bc.RED, f'    WANRING: There are {num_missing} states between given `emin`: {emin} and lowest `dis_froz_min`: {min(froz_min_list)}. \n' \
+                                   '    Please carefully consider the suggestion of dis frozen window and double-check energy range of each band.\n' \
+                                   '    This is a frequent occurrence in no-SOC systems with numerous denegeracy point.\n' \
+                                   '    However, we still want to provide you with some alternative energy window options.\n')
                 for i in range(self.nwann - num_missing, self.nwann, 1):
                     emax_i = self.suggest_froz_max(emin, nwann=i)
                     new = pd.DataFrame({"dis_froz_min" : emin,
@@ -423,9 +443,9 @@ class W90():
 
     def edit_win(self, dis_dict:Dict[str,float]):
         r'''
-        Edit `self._sys.win` file with input dis windows from `dis_dict`.
+        Edit `self.seedname.win` file with input dis windows from `dis_dict`.
         '''
-        with open(os.path.join(self.path, self._win), 'r+') as file:
+        with open(os.path.join(self.path, self.win), 'r+') as file:
             lines = file.readlines()
 
         for idx, line in enumerate(lines):
@@ -435,7 +455,7 @@ class W90():
                 if m:
                     lines[idx] = '{0:<17s} = {1}\n'.format(m.group(0), dis_dict[key])
 
-        with open(os.path.join(self.path, self._win), 'w+') as file:
+        with open(os.path.join(self.path, self.win), 'w+') as file:
             file.writelines(lines)
 
     def update_bands(self):
@@ -448,8 +468,8 @@ class W90():
         self.w2v_ratio      : ratio of k-distance. Theoretically, it should be 2 * pi or 1
         self.nbnds_excl     : Number of VASP bands below the lowest Wannier90 band
         '''
-        self.vkk, self.vee = self._parse_dat(os.path.join(self.path, self.config.vasp_bnd))
-        self.wkk, self.wee = self._parse_dat(os.path.join(self.path, self.config.w90_bnd))
+        self.vkk, self.vee = self._parse_dat(os.path.join(self.path, self.vasp_bnd))
+        self.wkk, self.wee = self._parse_dat(os.path.join(self.path, self.w90_bnd))
         self.w2v_ratio = np.max(self.vkk) / np.max(self.wkk)   # Theoretically, it should be 2 * pi or 1
 
         # Use eigenvalues in first k-point to align W90 data and VASP data
@@ -460,10 +480,10 @@ class W90():
 
     def plot_cmp_vasp_w90(self, name:str, ylim=None, font="Open Sans", size=18):
         r'''
-        Plot VASP band and Wannier90 band with legend. The file name is `{name}_VASP_W90_cmp.png`
+        Plot VASP band and Wannier90 band with legend. The file name is `{name}_VASP_W90_cmp.pdf`
         '''
         self.update_bands()
-        output_figure = os.path.join(self.path, f'{name}_VASP_W90_cmp.png')
+        output_figure = os.path.join(self.path, f'{name}_VASP_W90_cmp.pdf')
 
         # general options for plot
         plt.rcParams['font.family'] = font
@@ -489,12 +509,20 @@ class W90():
         with open(labelinfo, 'r') as f:
             lines = f.readlines()
         label = [l.split()[0] for l in lines]
-        logger.info(f"k label: {label}")
-        k_node = [eval(l.split()[2]) for l in lines]
+        # print(f"k label: {label}")
+        k_node = np.array([eval(l.split()[2]) for l in lines])
         for i, lab in enumerate(label):
             if lab[-1].isdigit():
                 label[i] = lab[:-1] + r'$_' + lab[-1] + r'$'
-    
+        k_node, idx = np.unique(k_node, return_index=True)
+        # merge the kpoint
+        new_label = []
+        idx = list(idx) + [None]
+        for i, j in zip(idx[:len(idx)-1], idx[1:]):
+            lab = label[i] if j == None or j - i == 1 else '|'.join(label[i:j])
+            new_label.append(lab)
+        label = new_label
+
         ax.set_xlim(k_node[0], k_node[-1])
         # put tickmarks and labels at node positions
         ax.set_xticks(k_node)
@@ -522,8 +550,9 @@ class W90():
                   prop={'size': 14})
 
         # plt.show()
-        # plt.savefig(output_figure,  bbox_inches='tight', transparent=True, dpi=300)
-        plt.savefig(output_figure,  bbox_inches='tight', dpi=300)
+        # plt.savefig(output_figure, bbox_inches='tight', transparent=True, dpi=300)
+        # plt.savefig(output_figure, bbox_inches='tight', dpi=300)
+        plt.savefig(output_figure, bbox_inches='tight')
 
     def evaluate(self, mode:str='AbAk', kernel=None) -> float:
         r"""
@@ -587,7 +616,7 @@ class W90():
         r'''
         Return spread message from `.wout` file.
         '''
-        conv_str = os.popen(f"grep CONV {os.path.join(self.path, self._sys +'.wout')}").read().split('\n')
+        conv_str = os.popen(f"grep CONV {os.path.join(self.path, self.seedname +'.wout')}").read().split('\n')
         if len(conv_str) > 4:
             conv = conv_str[3:-1]
             spread = np.array([eval(c.split()[3]) for c in conv])
@@ -621,9 +650,42 @@ class W90():
             d[k] = opt_input[i]
         return d
 
+    def check_dis_inner_dict(self, opt_input:ArrayLike) -> bool:
+        r"""
+        Check input value. When number of states inside frozen windows is greater than number of WFs,
+        the function will return `False` with WARNING.
+
+        - `opt_input`: List of all energy values to be optimized with default order in `self.opt_ini_dis`.
+        """
+        dis = self.total_dis_dict(opt_input)
+        n_froz = self.count_states_most((dis['dis_froz_min'], dis['dis_froz_max']))
+        if n_froz > self.nwann:
+            logger.warning(f"There are {n_froz:3d} states inside frozen window but {self.nwann:3d} WFs given!".ljust(80, ' '))
+            logger.warning("Fix `dis windows` as following".ljust(80, ' '))
+            return False
+        else:
+            return True
+
+    def check_dis_outer_dict(self, opt_input:ArrayLike) -> bool:
+        r"""
+        Check input value. When number of states inside dis windows is less than number of WFs,
+        the function will return `False` with WARNING.
+
+        - `opt_input`: List of all energy values to be optimized with default order in `self.opt_ini_dis`.
+        """
+        dis = self.total_dis_dict(opt_input)
+        n_win  = self.count_states_least((dis['dis_win_min'], dis['dis_win_max']))
+        if n_win < self.nwann:
+            logger.warning(f"There are {n_win:3d} states inside dis window but {self.nwann:3d} WFs given!".ljust(80, ' '))
+            logger.warning("Fix `dis windows` as following".ljust(80, ' '))
+            return False
+        else:
+            return True
+
     def check_total_dis_dict(self, opt_input:ArrayLike) -> bool:
         r"""
-        Check input value. When number of states inside frozen windows is greater than number of WFs or number of states inside dis windows is less than number of WFs, the function will return `False` with WARNING.
+        Check input value. When number of states inside frozen windows is greater than number of WFs or number of states inside dis windows is less than number of WFs,
+        the function will return `False` with WARNING.
 
         - `opt_input`: List of all energy values to be optimized with default order in `self.opt_ini_dis`.
         """
@@ -631,10 +693,12 @@ class W90():
         n_froz = self.count_states_most((dis['dis_froz_min'], dis['dis_froz_max']))
         n_win  = self.count_states_least((dis['dis_win_min'], dis['dis_win_max']))
         if n_froz > self.nwann:
-            logger.warning(f"There are {n_froz:3d} states inside frozen window but {self.nwann:3d} WFs given! \nFix `dis windows` as following".ljust(80, ' '))
+            logger.warning(f"There are {n_froz:3d} states inside frozen window but {self.nwann:3d} WFs given!".ljust(80, ' '))
+            logger.warning("Fix `dis windows` as following".ljust(80, ' '))
             return False
         if n_win < self.nwann:
-            logger.warning(f"There are {n_win:3d} states inside dis window but {self.nwann:3d} WFs given!\nFix `dis windows` as following".ljust(80, ' '))
+            logger.warning(f"There are {n_win:3d} states inside dis window but {self.nwann:3d} WFs given!".ljust(80, ' '))
+            logger.warning("Fix `dis windows` as following".ljust(80, ' '))
             return False
         return True
 
@@ -659,16 +723,21 @@ class W90():
 
     def rational_opt_input(self, opt_input:ArrayLike) -> Dict[str,float]:
         r'''
-        Return rationalized full dis windows dict from input. `Wannier90` requires at most #WFs of states inside the frozen energy window and at least #WFs of states inside the dis energy window. We treat the input `dis_win_min` as beginning to generate `dis_froz_max`, `dis_froz_min` and `dis_win_max` which satisifying the requirement of `Wannier90`.
+        Return rationalized full dis windows dict from input. `Wannier90` requires at most #WFs of states inside the frozen energy window
+        and at least #WFs of states inside the dis energy window. 
+        We treat the input `dis_win_min` as beginning to generate `dis_froz_max`, `dis_froz_min` and `dis_win_max` 
+        which satisifying the requirement of `Wannier90`.
         
         - `opt_input`: List of all energy values to be optimized with default order in `self.opt_ini_dis`.
         '''
         dis = self.total_dis_dict(opt_input)
         emin = dis['dis_win_min']
-        emax = self.suggest_froz_max(emin)
-        dis['dis_froz_max'] = emax
-        dis['dis_froz_min'] = self.suggest_froz_min(emax)
-        dis['dis_win_max']  = self.suggest_win_max(emin)
+        if not self.check_dis_inner_dict(opt_input):
+            emax = self.suggest_froz_max(emin)
+            dis['dis_froz_max'] = emax
+            dis['dis_froz_min'] = self.suggest_froz_min(emax)
+        if not self.check_dis_outer_dict(opt_input):
+            dis['dis_win_max']  = self.suggest_win_max(emin)
         return [dis[k] for k in self.config.opt_ini_dis.keys()]
 
     def show_spread(self, update:bool=False, terminal:bool=False):
@@ -687,7 +756,7 @@ class W90():
         len_unit_spread = max(spread / 20)
         len_spread = (spread / len_unit_spread).astype(int)
         N2 = (len(spread) + 1)// 2
-        lines.append(f"Spread (Ang^2) in `{self._sys}.wout`:".ljust(80, ' '))
+        lines.append(f"Spread (Ang^2) in `{self.seedname}.wout`:".ljust(80, ' '))
         lines.append(f"+-----+{'-'*32}++-----+{'-'*32}+")
         lines.append(f"|   i |{' Spread':32s}||   i |{' Spread':32s}|")
         lines.append(f"+-----+{'-'*32}++-----+{'-'*32}+")
@@ -728,7 +797,7 @@ class W90():
         lines.append(f"+-----+{'-'*32}++-----+{'-'*32}+")
         lines.append(f"|   i |{' MAX DIFF (meV)':32s}||   i |{' AVERAGE DIFF (meV)':32s}|")
         lines.append(f"+-----+{'-'*32}++-----+{'-'*32}+")
-        for i in range(self.nwann):
+        for i in range(len(self.dEs)):
             max_str_i = f'{self._block * len_dEs_max[i]} {dEs_max[i]:.2f}'
             ave_str_i = f'{self._block * len_dEs_ave[i]} {dEs[i]:.2f}'
             lines.append(f'|{i+1:4d} |{max_str_i:<32s}||{i+1:4d} |{ave_str_i:<32s}|')
