@@ -5,6 +5,67 @@ import os, re, copy
 
 from pymatgen.core import structure
 
+class bc:
+    r"""
+    Module for colorful print.
+
+    - Title, important messages (comments, wanrings etc.): `cprint` with `bc.RED`
+    - menu, selection, comments, system messages: `cprint` with `bc.BLUE`
+    - data output: `print`
+
+    ### Method
+    `cprint`
+    """
+    END       = '\33[0m'
+    BOLD      = '\33[1m'
+    ITALIC    = '\33[3m'
+    URL       = '\33[4m'
+    BLINK     = '\33[5m'
+    BLINK2    = '\33[6m'
+    SELECTED  = '\33[7m'
+    BLACK     = '\33[30m'
+    RED       = '\33[31m'
+    GREEN     = '\33[32m'
+    YELLOW    = '\33[33m'
+    BLUE      = '\33[34m'
+    VIOLET    = '\33[35m'
+    BEIGE     = '\33[36m'
+    WHITE     = '\33[37m'
+    BLACKBG   = '\33[40m'
+    REDBG     = '\33[41m'
+    GREENBG   = '\33[42m'
+    YELLOWBG  = '\33[43m'
+    BLUEBG    = '\33[44m'
+    VIOLETBG  = '\33[45m'
+    BEIGEBG   = '\33[46m'
+    WHITEBG   = '\33[47m'
+    GREY      = '\33[90m'
+    RED2      = '\33[91m'
+    GREEN2    = '\33[92m'
+    YELLOW2   = '\33[93m'
+    BLUE2     = '\33[94m'
+    VIOLET2   = '\33[95m'
+    BEIGE2    = '\33[96m'
+    WHITE2    = '\33[97m'
+    GREYBG    = '\33[100m'
+    REDBG2    = '\33[101m'
+    GREENBG2  = '\33[102m'
+    YELLOWBG2 = '\33[103m'
+    BLUEBG2   = '\33[104m'
+    VIOLETBG2 = '\33[105m'
+    BEIGEBG2  = '\33[106m'
+    WHITEBG2  = '\33[107m'
+
+    @classmethod
+    def cprint(cls, color, *args, **kwargs):
+        r"""
+        Colorfun print using ANSI escape sequences.
+
+        `color`: defined color in `bc` class.
+        """
+        str_args = ' '.join(str(arg) for arg in args)
+        print(f'{color}{str(str_args)}{cls.END}', **kwargs)
+
 def gaussian(x:ArrayLike, mid:float= 0, width:float=3, normalized:bool=False) -> ArrayLike:
     r'''
     Return values of Gaussian function with given input. Used as kernel function for band fitting quality evaluation. 
